/*----------------------------------------------------------------------------
 * CMSIS-RTOS 'main' function template
 *---------------------------------------------------------------------------*/
 #include "stm32f7xx_hal.h"
#include "RTE_Components.h"
#include  CMSIS_device_header
#include "cmsis_os2.h"
#include "ext_led.h" 
#include "ext_uart.h"
#include "ext_buttons.h"
#include "ext_keyboard.h"
#include "string.h"
#include <stdio.h>
#include <stdlib.h>

#ifdef RTE_Compiler_EventRecorder
#include "EventRecorder.h"
#endif

osThreadId_t thread1,thread2;
osMessageQueueId_t msgQueue;

 const osThreadAttr_t thread1_attr = {
  .stack_size = 1024,          // Create the thread stack size 
	.priority = osPriorityNormal, //Set initial thread priority to high
	 .name = "Producer",
};
 const osThreadAttr_t thread2_attr = {
  .stack_size = 1024,          // Create the thread stack size 
	.priority = osPriorityNormal, //Set initial thread priority to high
	 .name = "Consumer",
};
 
const osMessageQueueAttr_t msgQueue_attr =  {
	.name = "MsgQueue",
};
 

//------------------------------------------------------------------------------
// Setup system clock to 216MHz
//------------------------------------------------------------------------------
void SystemClock_Config (void) {
  RCC_ClkInitTypeDef RCC_ClkInitStruct;
  RCC_OscInitTypeDef RCC_OscInitStruct;

  /* Enable HSE Oscillator and activate PLL with HSE as source */
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSE;
  RCC_OscInitStruct.HSEState = RCC_HSE_ON;
  RCC_OscInitStruct.HSIState = RCC_HSI_OFF;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
  RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSE;
  RCC_OscInitStruct.PLL.PLLM = 25;
  RCC_OscInitStruct.PLL.PLLN = 432;  
  RCC_OscInitStruct.PLL.PLLP = RCC_PLLP_DIV2;
  RCC_OscInitStruct.PLL.PLLQ = 9;
  HAL_RCC_OscConfig(&RCC_OscInitStruct);

  /* Activate the OverDrive to reach the 216 MHz Frequency */
  HAL_PWREx_EnableOverDrive();
  
  /* Select PLL as system clock source and configure the HCLK, PCLK1 and PCLK2 clocks dividers */
  RCC_ClkInitStruct.ClockType = (RCC_CLOCKTYPE_SYSCLK | RCC_CLOCKTYPE_HCLK | RCC_CLOCKTYPE_PCLK1 | RCC_CLOCKTYPE_PCLK2);
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV4;  
  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV2;  
  HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_7);
}



/*----------------------------------------------------------------------------
 * Thread Producer
 *---------------------------------------------------------------------------*/
__NO_RETURN static void Thread_Producer (void *argument) {
	osMessageQueueId_t* queue = (osMessageQueueId_t*)argument;
	uint32_t counter = 0;
	
	for(;;) {
		osStatus_t statusQueue = osMessageQueuePut(*queue, &counter, 1, 0);
		if(statusQueue != osOK) {
			// TODO generate error
		}
		counter++;
		osDelay(100);
	}

}

/*----------------------------------------------------------------------------
 * Thread Consumer
 *---------------------------------------------------------------------------*/
__NO_RETURN static void Thread_Consumer (void *argument) {
	osMessageQueueId_t* queue = (osMessageQueueId_t*)argument;
	uint8_t counter = 1;
	
	for(;;) {
		uint32_t msg;
		osStatus_t statusQueue;
		
		do {
			statusQueue = osMessageQueueGet(*queue, &msg, NULL, 0);
			printf("Value is %d\r\n", msg);
		} while(statusQueue != osErrorResource);
		if(++counter > 10) counter = 1;
		osDelay(counter*100);
	}
}
 

int main (void) {
 
  // System Initialization
	SystemClock_Config();
  SystemCoreClockUpdate();
#ifdef RTE_Compiler_EventRecorder
  // Initialize and start Event Recorder
//	Ext_UART_Init(9600);
  EventRecorderInitialize(EventRecordAll, 1U);
#endif
 
  osKernelInitialize();                 // Initialize CMSIS-RTOS
	msgQueue = osMessageQueueNew(8, 4, &msgQueue_attr);
  thread1 = osThreadNew(Thread_Producer, &msgQueue,  &thread1_attr);
  thread2 = osThreadNew(Thread_Consumer, &msgQueue,  &thread2_attr);
	
	//----------------------------------------------------------------------------------------------
	// get names are placed for TraceAlyzer visualisation
	//----------------------------------------------------------------------------------------------
	osThreadGetName(thread1);
	osThreadGetName(thread2);
	osMessageQueueGetName(msgQueue);
  osKernelStart();                      // Start thread execution
  for (;;) {}
}
